{-----------------------------------------------------------------------}
{           Subprogram Name:  AdoIniFile                                }
{           Purpose:          Hold ADO connection information           }
{           Source Language:  Delphi                                    }
{           Author Name and Contact Information:                        }
{           By Greg Nixon                                               }
{           greg@nzcs.co.nz                                             }
{-----------------------------------------------------------------------}
{           Description:                                                }
{           AdoIniFile allows you to create/use an INI                  }
{           file that is compressed                                     }
{-----------------------------------------------------------------------}


{$I NZCS.INC}  { Standard defines for all Delphi Free Stuff components }

unit NZCSADOini;

interface

uses Windows, SysUtils, Classes, IniFiles, NZCSCompress;

const
   CT_PROVIDER    = 'PROVIDER=';
   CT_PASSWORD    = 'PASSWORD=';
   CT_USERID      = 'USER ID=';
   CT_CATALOG     = 'INITIAL CATALOG=';
   CT_DATASOURCE  = 'DATA SOURCE=';

type
   TADOIniFile = class(TCustomIniFile)
   private
      FFileName: String;
      FSections: TStringList;
      function AddSection(const Section: string): TStrings;
      function GetAliasName(const Index: Integer): string;
   protected
   public
      constructor Create(const FileName: String);
      destructor Destroy; override;
      procedure LoadFile;
      procedure Clear;
      procedure DeleteKey(const Section, Ident: string); override;
      procedure EraseSection(const Section: string); override;
      procedure RenameAlias(OldName, NewName: String);
      procedure GetStrings(List: TStrings);
      procedure ReadSection(const Section: string; Strings: TStrings); override;
      procedure ReadFullSection(const Section: string; Strings: TStrings);
      procedure ReadSections(Strings: TStrings); override;
      procedure ReadSectionValues(const Section: string; Strings: TStrings); override;
      function ReadString(const Section, Ident, default: string): string; override;
      procedure Rename(const FileName: string; Reload: Boolean);
      function  SectionExists(Const Section: String): Boolean;
      procedure SetStrings(List: TStrings);
      procedure UpdateFile; override;
      procedure WriteString(const Section, Ident, Value: string); override;
      function AliasCount: Integer;
      property  AliasName[const Index: Integer]: string read GetAliasName;
      property FileName: string read FFileName;
   end;


   TADOAlias = class(TADOIniFile)
   private

   public
      procedure AddAlias(const Alias, ConnectionString, Description,
                         Application, Custom1, Custom2: String);
      procedure ModifyAlias(const Alias, ConnectionString, Description,
                                  Application, Custom1, Custom2: String);

      procedure Add(const Alias: String; the_params: TStrings);
      procedure Delete(const Alias: String);
      function  Exists(const Alias: String): Boolean;
      procedure Modify(const Alias: String; the_params: TStrings);
      procedure GetAliasList(AliasList: TStrings);
      procedure GetParams(const alias: String; the_params: TStrings);
      function GetParamValue(const alias: String; Const field: String): String;

      function GetProvider(const alias: String): String;
      function GetDataSource(const alias: String): String;
      function GetUserID(const alias: String): String;
      function GetPassword(const alias: String): String;
      function GetCatalog(const alias: String): String;

      procedure AddConnectionStr(const Alias: String; ConnectionString: String);
      function GetConnectionStr(const Alias: String): String;
   end;

function ExtractConnectionTag(ConnStr, Tag: string): string;
function ReadUDL(Filename: String): String;
function GetNewADOAlias: TADOAlias;

implementation

uses NZCSFunct, NZCSStringList, NZCSadoConst, FileCtrl;

// Utility //

function GetNewADOAlias: TADOAlias;
var
   Fn: String;
begin
   Fn := GetRegStrValue(HKEY_CURRENT_USER,REGKey,'ADOConfig');
   If (Fn = '') or not FileExists(Fn) then begin
      Fn := ExtractFileDrive(FindWindowsDir) + 'Program Files\Common Files\ADOAdmin\';
      ForceDirectories(FN);
      FN := FN + 'ADOConfig.dcs';
      result := TADOAlias.Create(fn);
      SetRegStrValue(HKEY_CURRENT_USER,REGKey,'ADOConfig', fn);
   end
   else
      result := TADOAlias.Create(fn);
end;



Function ReadUDL(Filename: String): String;
var
   UDLList: TUStringList;
   ConStr: String;
   i: Integer;
   FirstChar: Char;
begin
   result := '';
   If FileExists(FileName) then begin
      UDLList := TUStringList.Create;
      Try
         UDLList.LoadFromFile(Filename);
         For i := 0 to UDLList.Count - 1 do begin
            FirstChar := AsChar(TrimLeft(UDLList[i]));
            If not (FirstChar in ['[',';']) then
               ConStr := ConStr + Trim(UDLList[i]);
         end;
      finally
         UDLList.Free;
         result := ConStr;
      end;                  
   end;
end;

function ExtractConnectionTag(ConnStr, Tag: string): string;
var
 TAGPos, TAGLen: Integer;
begin
   TAGPos := Pos(Tag, AnsiUpperCase(ConnStr)) + Length(Tag);
   if TAGPos > Length(Tag) then begin
      TAGLen := Pos(';', Copy(ConnStr, TAGPos, 255)) - 1;
      If TAGLen < 0 then
         TAGLen := MaxInt;
      Result := Copy(ConnStr, TAGPos, TAGLen);
   end
   else
      Result := '';
end;

Procedure ConnectionStrToStringList(const Value: string; MyList: TStringList);
var
  P, P1: PChar;
  S: string;
begin
   With MyList do begin
     BeginUpdate;
     try
       Clear;
       P := PChar(Value);
       while P^ in [#1..#31] do P := CharNext(P);
       while P^ <> #0 do begin
          P1 := P;
          while (P^ > #31) and (P^ <> ';') do P := CharNext(P);
          SetString(S, P1, P - P1);
          Add(Trim(S));
          while P^ in [#1..#31] do P := CharNext(P);
          if P^ = ';' then begin
             repeat
                P := CharNext(P);
             until not (P^ in [#1..#31]);
          end;
       end;
     finally
       EndUpdate;
     end;
   end;
end;

{$IFNDEF NZCS_COMPILER_4_UP}
function SameText(StrOne, StrTwo: String): Boolean;
begin
   result := (CompareText(StrOne, StrTwo) = 0);
end;
{$ENDIF}

constructor TADOIniFile.Create(const FileName: String);
begin
   inherited Create(FileName);
   FSections := TStringList.Create;
   FFileName := FileName;
   LoadFile;
end;

destructor TADOIniFile.Destroy;
begin
   if FSections <> nil then Clear;
   FSections.Free;
end;

function TADOIniFile.AddSection(const Section: string): TStrings;
begin
   Result := TStringList.Create;
   try
      FSections.AddObject(Section, Result);
   except
      Result.Free;
   end;
end;

procedure TADOIniFile.Clear;
var
   I: Integer;
begin
   for I := 0 to FSections.Count - 1 do
      TStrings(FSections.Objects[I]).Free;
   FSections.Clear;
end;

procedure TADOIniFile.DeleteKey(const Section, Ident: string);
var
   I, J: Integer;
   Strings: TStrings;
begin
   I := FSections.IndexOf(Section);
   if I >= 0 then begin
      Strings := TStrings(FSections.Objects[I]);
      J := Strings.IndexOfName(Ident);
      if J >= 0 then Strings.Delete(J);
   end;
end;

procedure TADOIniFile.EraseSection(const Section: string);
var
   I: Integer;
begin
   I := FSections.IndexOf(Section);
   if I >= 0 then begin
      TStrings(FSections.Objects[I]).Free;
      FSections.Delete(I);
   end;
end;

procedure TADOIniFile.GetStrings(List: TStrings);
var
   I, J: Integer;
   Strings: TStrings;
begin
   List.BeginUpdate;
   try
      for I := 0 to FSections.Count - 1 do begin
         List.Add('[' + FSections[I] + ']');
         Strings := TStrings(FSections.Objects[I]);
         for J := 0 to Strings.Count - 1 do List.Add(Strings[J]);
         List.Add('');
      end;
   finally
      List.EndUpdate;
   end;
end;

procedure TADOIniFile.LoadFile;
var
   List: TStringList;
   CStream: TFileStream;
   PStream: TMemoryStream;
   SStream: TMemoryStream;
   LHACompression: TLHACompression;
   Success: Boolean;
begin
   if (FFileName <> '') and FileExists(FFileName) then begin
      List := TStringList.Create;
      Clear;
      try
         CStream:= TFileStream.Create(FFileName, fmOpenRead or fmShareDenyWrite);
         LHACompression := TLHACompression.Create;
         SStream := TMemoryStream.Create;
         LHACompression.LHAExpand(CStream, SStream);
         CStream.Free;
         SStream.Position := 0;
         List.LoadFromStream(SStream);
         SStream.Free;
         LHACompression.Free;
         SetStrings(List);
      finally
         List.Free;
      end;
   end
   else
      Clear;
end;

procedure TADOIniFile.ReadSection(const Section: string;
Strings: TStrings);
var
   I, J: Integer;
   SectionStrings: TStrings;
begin
   Strings.BeginUpdate;
   try
      Strings.Clear;
      I := FSections.IndexOf(Section);
      if I >= 0 then begin
         SectionStrings := TStrings(FSections.Objects[I]);
         for J := 0 to SectionStrings.Count - 1 do
            Strings.Add(SectionStrings.Names[J]);
      end;
   finally
      Strings.EndUpdate;
   end;
end;

procedure TADOIniFile.ReadFullSection(const Section: string;
Strings: TStrings);
var
   I, J: Integer;
   SectionStrings: TStrings;
begin
   Strings.BeginUpdate;
   try
      Strings.Clear;
      I := FSections.IndexOf(Section);
      if I >= 0 then begin
         SectionStrings := TStrings(FSections.Objects[I]);
         for J := 0 to SectionStrings.Count - 1 do
            Strings.Add(SectionStrings[J]);
      end;
   finally
      Strings.EndUpdate;
   end;
end;

procedure TADOIniFile.ReadSections(Strings: TStrings);
begin
   Strings.Assign(FSections);
end;

function TADOIniFile.SectionExists(Const Section: String): Boolean;
begin
   result := (FSections.IndexOf(Section) > -1);
end;

procedure TADOIniFile.ReadSectionValues(const Section: string;
Strings: TStrings);
var
   I: Integer;
begin
   Strings.BeginUpdate;
   try
      Strings.Clear;
      I := FSections.IndexOf(Section);
      if I >= 0 then Strings.Assign(TStrings(FSections.Objects[I]));
   finally
      Strings.EndUpdate;
   end;
end;

function TADOIniFile.ReadString(const Section, Ident, default: string): string;
var
   I: Integer;
   Strings: TStrings;
begin
   I := FSections.IndexOf(Section);
   if I >= 0 then begin
      Strings := TStrings(FSections.Objects[I]);
      I := Strings.IndexOfName(Ident);
      if I >= 0 then begin
         Result := Copy(Strings[I], Length(Ident) + 2, Maxint);
         Exit;
      end;
   end;
   Result := default;
end;

procedure TADOIniFile.Rename(const FileName: string; Reload: Boolean);
begin
   FFileName := FileName;
   if Reload then LoadFile;
end;

procedure TADOIniFile.SetStrings(List: TStrings);
var
   I: Integer;
   S: string;
   Strings: TStrings;
begin
   Clear;
   Strings := nil;
   for I := 0 to List.Count - 1 do begin
      S := List[I];
      if (S <> '') and (S[1] <> ';') then
         if (S[1] = '[') and (S[Length(S)] = ']') then
         Strings := AddSection(Copy(S, 2, Length(S) - 2))
      else
         if Strings <> nil then Strings.Add(S);
   end;
end;

procedure TADOIniFile.UpdateFile;
var
   List: TStringList;
   CStream: TFileStream;
   PStream: TMemoryStream;
   SStream: TMemoryStream;
   LHACompression: TLHACompression;
begin
   List := TStringList.Create;
   try
      GetStrings(List);
      If FileExists(FFileName) then
         DeleteFile(Pchar(FFileName));
      CStream := TFileStream.Create(FFileName, fmCreate);
      LHACompression := TLHACompression.Create;
      SStream := TMemoryStream.Create;
      PStream := TMemoryStream.Create;
      Try
         List.SaveToStream(SStream);
         SStream.Position := 0;
         LHACompression.LHACompress(SStream, PStream);
         PStream.Position := 0;
         CStream.CopyFrom(PStream,0);
      finally
         CStream.Free;
         PStream.Free;
         SStream.Free;
         LHACompression.Free;
      end;
   finally
      List.Free;
   end;
end;

procedure TADOIniFile.WriteString(const Section, Ident, Value: string);
var
   I: Integer;
   S: string;
   Strings: TStrings;
begin
   I := FSections.IndexOf(Section);
   if I >= 0 then
      Strings := TStrings(FSections.Objects[I])
   else
      Strings := AddSection(Section);
   S := Ident + '=' + Value;
   I := Strings.IndexOfName(Ident);
   if I >= 0 then
      Strings[I] := S
   else
      Strings.Add(S);
end;

Procedure TADOIniFile.RenameAlias(OldName, NewName: String);
var
   i: integer;
begin
   I := FSections.IndexOf(OldName);
   If I > -1 then FSections[I] := NewName;
end;



{ TADOAlias }

procedure TADOAlias.AddAlias(const Alias, ConnectionString, Description,
                                   Application, Custom1, Custom2: String);
begin
   EraseSection(Alias);
   ModifyAlias(Alias, ConnectionString, Description,
                      Application, Custom1, Custom2);
end;

procedure TADOAlias.ModifyAlias(const Alias, ConnectionString, Description,
                                   Application, Custom1, Custom2: String);
begin
   if Description <> '' then
      WriteString(Alias, 'Description', Description);
   if Application <> '' then
      WriteString(Alias, 'Application', Application);
   if Custom1 <> '' then
      WriteString(Alias, 'Custom1', Custom1);
   if Custom2 <> '' then
      WriteString(Alias, 'Custom2', Custom2);
   AddConnectionStr(Alias,ConnectionString);
end;

procedure TADOAlias.Add(const Alias: String; the_params: TStrings);
begin
   EraseSection(Alias);
   Modify(Alias, the_params);
end;

procedure TADOAlias.AddConnectionStr(const Alias: String; ConnectionString: String);
var
   TmpList: TStringList;
   i: Integer;
   Fld: String;
begin
   TmpList := TStringList.Create;
   Try
      ConnectionStrToStringList(ConnectionString, TmpList);
      For i := 0 to TmpList.Count - 1 do begin
         Fld := TmpList.Names[i];
         WriteString(Alias, Fld, TmpList.Values[Fld]);
      end;
   finally
      TmpList.Free;
   end;
end;

function TADOAlias.GetConnectionStr(const Alias: String): String;
var
   TmpList: TStringList;
   i: Integer;
   Fld,Ans: String;
begin
   TmpList := TStringList.Create;
   Try
      ReadSection(Alias,TmpList);
      Ans := '';
      For i := 0 to TmpList.Count - 1 do begin
         Fld := TmpList[i];
         if not SameText(Fld,'DESCRIPTION') and
            not SameText(Fld,'APPLICATION') and
            not SameText(Fld,'CUSTOM1') and
            not SameText(Fld,'CUSTOM2') then
            Ans := Ans + Fld + '=' + ReadString(Alias, Fld,'') + ';';
      end;
      If Ans <> '' then
         System.Delete(Ans,Length(Ans),1);
      result := Ans;
   finally
      TmpList.Free;
   end;
end;

procedure TADOAlias.Delete(const Alias: String);
begin
   EraseSection(Alias);
end;

function TADOAlias.Exists(const Alias: String): Boolean;
begin
   result := SectionExists(Alias);
end;

procedure TADOAlias.GetAliasList(AliasList: TStrings);
begin
   ReadSections(AliasList);
end;

function TADOAlias.GetCatalog(const alias: String): String;
begin
   result := GetParamValue(alias, CT_CATALOG);
end;

function TADOAlias.GetDataSource(const alias: String): String;
begin
  result := GetParamValue(alias, CT_DATASOURCE);
end;

procedure TADOAlias.GetParams(const alias: String; the_params: TStrings);
begin
   ReadFullSection(Alias,the_params);
end;

function TADOAlias.GetParamValue(const alias: string; const field: String): String;
begin
   result := ReadString(alias, field, '');
end;

function TADOAlias.GetPassword(const alias: String): String;
begin
   result := GetParamValue(alias, CT_PASSWORD);
end;

function TADOAlias.GetProvider(const alias: String): String;
begin
   result := GetParamValue(alias, CT_PROVIDER);
end;

function TADOAlias.GetUserID(const alias: String): String;
begin
   result := GetParamValue(alias, CT_USERID);
end;

procedure TADOAlias.Modify(const Alias: String; the_params: TStrings);
var
   fld: String;
   i: integer;
begin
   for i := 0 to the_params.Count -1 do begin
      Fld := the_params.Names[i];
      WriteString(Alias, Fld, the_params.Values[Fld]);
   end;
end;


function TADOIniFile.GetAliasName(const Index: Integer): string;
begin
   If (Index > -1) and (Index <= FSections.Count) then
      result := FSections[Index]
   else
      result := '';
end;

function TADOIniFile.AliasCount: Integer;
begin
   result := FSections.Count;
end;

end.

